package it.holhoconverter;

import it.holhoconverter.converter.ConversionPack;
import it.holhoconverter.converter.FFMpegLogger;
import it.holhoconverter.converter.VideoConverter;
import it.holhoconverter.frames.ILangComponent;
import it.holhoconverter.frames.MainFrame;
import it.holhoconverter.frames.OptionsFrame;
import it.holhoconverter.language.LangManager;
import it.holhoconverter.utils.DirUtils;
import it.holhoconverter.utils.LinkLabel;
import it.holhoconverter.utils.Logger;
import it.holhoconverter.utils.DirUtils.OS;

import java.awt.EventQueue;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import com.apple.eawt.Application;

public class HolhoConverter
{
	/**
	 * @author Vincenzo Fortunato e-mail: "planetgeekita@gmail.com" - website:
	 *         "www.planetgeeks.org"
	 */

	public static HolhoConverter instance;

	/** RESOURCES PATHS & NAMES **/
	public static String LANGS_PKG = "langs/";
	public static String ICONS_PKG = "icons/";
	public static String RES_PKG = "resources/";
	public static String LANGS_LIST = "languages.list";
	public static String SETTINGS;
	public static String DEF_OUTPUT_VIDEODIR;
	public static String LOG_FOLDER;
	public static String LOG_FILE_PREFIX;
	public static String FFMPEG_LOG_FOLDER;
	public static String FFMPEG_LOG_FILE_PREFIX;
	public static String DATA_DIR;
	public static String UPDATE_CHECK;

	/** SETTINGS **/
	public static String KEY_LANG = "lang";
	public static String KEY_LAST_DIR = "last_open_dir";
	public static String KEY_SAVE_DIR = "save_dir";
	public static String KEY_QUALITY = "quality";
	public static String KEY_FORMAT = "format";
	public static String KEY_CONV_MODE = "c_mode";
	public static String KEY_AUDIO = "active_audio";
	public static String KEY_BITRATE = "bit_rate";
	public static String KEY_SHOWUPDATE = "showUpdate";
	public static int MAX_LOGS;
	public static int FFMPEG_LOGS;
	public static int TABLE_CAPACITY;

	/** UTILS **/
	public static int FFMPEG_PROCESS_COUNT = 0;
	public boolean updating = false;

	public Settings settings;
	public LangManager langManager;
	public MainFrame mainFrame;
	public OptionsFrame optionsFrame;
	public ColorManager colors;
	public boolean updateMessage = true;

	private ArrayList<ILangComponent> iLangComponents = new ArrayList<ILangComponent>();

	private void init()
	{
		settings = new Settings();

		DATA_DIR = settings.getInternalSettingsValue("DATA_DIR");
		DEF_OUTPUT_VIDEODIR = settings.getInternalSettingsValue("DEF_OUTPUT_VIDEODIR");
		LOG_FOLDER = settings.getInternalSettingsValue("LOG_FOLDER");
		LOG_FILE_PREFIX = settings.getInternalSettingsValue("LOG_FILE_PREFIX");
		FFMPEG_LOG_FOLDER = settings.getInternalSettingsValue("FFMPEG_LOG_FOLDER");
		FFMPEG_LOG_FILE_PREFIX = settings.getInternalSettingsValue("FFMPEG_LOG_FILE_PREFIX");
		MAX_LOGS = Integer.valueOf(settings.getInternalSettingsValue("MAX_LOGS"));
		FFMPEG_LOGS = Integer.valueOf(settings.getInternalSettingsValue("FFMPEG_LOGS"));
		TABLE_CAPACITY = Integer.valueOf(settings.getInternalSettingsValue("TABLE_CAPACITY"));
		SETTINGS = settings.getInternalSettingsValue("SETTINGS_FILE");
		UPDATE_CHECK = settings.getInternalSettingsValue("UPDATE_CHECK");

		Logger.loadLogger();

		FFMpegLogger.loadLogger();

		colors = new ColorManager();

		loadLookAndFeel();

		settings.loadLocalSettings();

		langManager = new LangManager();

		updateMessage = settings.getInternalSettingsValue("UPDATE_MESSAGE").equals("true");

		if (updateMessage)
			checkForUpdates(true);

		initAndShowFrames();
	}

	public void checkForUpdates(boolean startUp)
	{
		if (settings.getValue(KEY_SHOWUPDATE) == null)
			settings.putValue(KEY_SHOWUPDATE, settings.getInternalSettingsValue("UPDATE_ONSTARTUP"));

		String show = settings.getValue(KEY_SHOWUPDATE);
		
		if (show == null || show.equals("true") || !startUp)
		{
			try
			{
				URL url = new URL(UPDATE_CHECK);
				BufferedReader br = new BufferedReader(new InputStreamReader(url.openStream()));
				String readed;
				while ((readed = br.readLine()) != null)
				{
					if (readed.startsWith("version="))
					{
						String version = readed.split("=")[1];
						String current = settings.getInternalSettingsValue("VERSION");
						if (!version.equals(current))
						{
							JCheckBox box = new JCheckBox(LangManager.getTranslated("update.check"));
							box.setSelected(show.equals("false"));
							Object[] components = { LangManager.getTranslated("update.message"), new LinkLabel(LangManager.getTranslated("update.link"), settings.getInternalSettingsValue("UPDATE_LINK")), box };
							JOptionPane.showMessageDialog(null, components, LangManager.getTranslated("update.title"), JOptionPane.INFORMATION_MESSAGE);
							settings.putValue(KEY_SHOWUPDATE, box.isSelected() ? "false" : "true");
						}
					}
				}
				br.close();
			}
			catch (MalformedURLException e)
			{
				Logger.log(Logger.GRAVE, "MalformedURLException occured during updates check!");
			}
			catch (IOException e)
			{
				Logger.log(Logger.GRAVE, "IOException occured during updates check!");
				e.printStackTrace();
			}
		}

	}

	private void initAndShowFrames()
	{

		if (DirUtils.getPlatform() == OS.macos)
		{
			Application.getApplication().setDockIconImage(Resources.getIcon(HolhoConverter.ICONS_PKG + "icon.png").getImage());

		}

		EventQueue.invokeLater(new Runnable()
		{
			public void run()
			{
				mainFrame = new MainFrame();
				mainFrame.setVisible(true);
			}
		});

	}

	public void showOptions(ConversionPack pack, int row)
	{
		if (optionsFrame == null)
			optionsFrame = new OptionsFrame();

		optionsFrame.show(pack, row);
	}

	public static void main(String[] args)
	{
		System.setProperty("com.apple.mrj.application.apple.menu.about.name", "Holho Converter");

		instance = new HolhoConverter();
		instance.init();
	}

	public void registerLangComponent(ILangComponent comp)
	{
		iLangComponents.add(comp);
	}

	public void updateLanguages()
	{
		for (ILangComponent comp : iLangComponents)
		{
			comp.updateLanguage();
		}
	}

	private void loadLookAndFeel()
	{
		try
		{
			for (UIManager.LookAndFeelInfo info : UIManager.getInstalledLookAndFeels())
			{
				if ("Nimbus".equals(info.getName()))
				{
					UIManager.setLookAndFeel(info.getClassName());
					UIManager.getLookAndFeelDefaults().put("nimbusOrange", colors.nimbusOrange);
					UIManager.put("nimbusBlueGrey", colors.nimbusBlueGrey);
					UIManager.put("nimbusBase", colors.nimbusBase);
					UIManager.put("control", colors.control);
					UIManager.put("nimbusLightBackground", colors.nimbusLightBackground);
					UIManager.put("nimbusFocus", colors.nimbusFocus);
					UIManager.put("text", colors.text);
					UIManager.put("nimbusDisabledText", colors.nimbusDisabledText);
					UIManager.put("textForeground", colors.textForeground);
					UIManager.put("nimbusSelectionBackground", colors.nimbusSelectionBackground);
					UIManager.put("Separator.foreground", colors.Separator_foreground);

					break;
				}
			}
		}
		catch (ClassNotFoundException ex)
		{
			java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		catch (InstantiationException ex)
		{
			java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		catch (IllegalAccessException ex)
		{
			java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
		catch (UnsupportedLookAndFeelException ex)
		{
			java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
	}

	public void exit()
	{
		VideoConverter.converting = false;

		VideoConverter.breakProcess();

		System.exit(0);
	}

	public void showPopupMessage(int type, String message, String title)
	{
		JOptionPane.showMessageDialog(null, message, title, type);
	}

}
