package it.holhoconverter;

import it.holhoconverter.utils.DirUtils;
import it.holhoconverter.utils.Logger;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Set;

public class Settings
{
	private File settings;
	private HashMap<String, String> settings_values = new HashMap<String, String>();
	private HashMap<String, String> internal_settings_values = new HashMap<String, String>();

	public Settings()
	{
		loadInternalSettings();
	}

	public void loadLocalSettings()
	{
		settings = new File(DirUtils.getWorkingDirectory() + File.separator + HolhoConverter.SETTINGS);

		if (settings.exists())
			loadSettings();
	}

	private void loadSettings()
	{
		try
		{
			BufferedReader br = new BufferedReader(new FileReader(settings));

			String line;
			while ((line = br.readLine()) != null)
			{
				if (line.contains("="))
				{
					String key = line.split("=")[0];
					String value = line.substring(key.length() + 1);

					settings_values.put(key, value);
				}
			}

			br.close();
		}
		catch (FileNotFoundException e)
		{
			Logger.log(Logger.GRAVE, "Error on loading settings! File " + settings.getAbsolutePath() + " doesn't exist");
			e.printStackTrace();
		}
		catch (IOException e)
		{
			Logger.log(Logger.GRAVE, "Error on loading settings! IOException occurred!");
			e.printStackTrace();
		}

		int setLoaded = settings_values.keySet() != null ? settings_values.keySet().size() : 0;

		Logger.log(Logger.INFO, "Loaded " + setLoaded + " settings!");
	}

	private void saveSettings()
	{
		try
		{
			BufferedWriter bw = new BufferedWriter(new FileWriter(settings));

			Set<String> keys = settings_values.keySet();

			boolean first = true;
			for (String key : keys)
			{
				if (!first)
					bw.newLine();
				bw.write(key + "=" + settings_values.get(key));
				first = false;
			}

			bw.close();
		}
		catch (IOException e)
		{
			Logger.log(Logger.GRAVE, "Error on saving settings!");
		}
	}

	public void putValue(String key, String value)
	{
		settings_values.put(key, value);

		saveSettings();
	}

	public String getValue(String key)
	{
		if (settings_values.containsKey(key))
			return settings_values.get(key);

		return null;
	}

	private void loadInternalSettings()
	{
		try
		{
			InputStream is = ClassLoader.getSystemClassLoader().getResourceAsStream("Settings");
			BufferedReader br = new BufferedReader(new InputStreamReader(is));

			String line;
			while ((line = br.readLine()) != null)
			{
				if (line.contains("="))
				{
					String key = line.split("=")[0];
					String value = line.substring(key.length() + 1);

					internal_settings_values.put(key, value);
				}
			}

			br.close();
			is.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}

	public String getInternalSettingsValue(String key)
	{
		if (internal_settings_values.containsKey(key))
			return internal_settings_values.get(key);

		return key;
	}

}
