package it.holhoconverter.converter;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.language.LangManager;
import it.holhoconverter.utils.DirUtils;
import it.holhoconverter.utils.DirUtils.OS;
import it.holhoconverter.utils.Logger;
import it.holhoconverter.utils.Utils;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.JOptionPane;

public class FFMpegProcess
{
	private ArrayList<String> commands;
	private Process process;
	private int id;
	private boolean muteLog = false;
	private HashMap<OS, String[]> ext_map = initializeExtMap();

	public FFMpegProcess()
	{
		commands = new ArrayList<String>();

		this.id = HolhoConverter.FFMPEG_PROCESS_COUNT++;
	}

	public void disableLog()
	{
		muteLog = true;
	}

	public FFMpegProcess(String... commandsArr)
	{
		this.commands = new ArrayList<String>();

		for (String command : commandsArr)
		{
			this.commands.add(command);
		}

		this.id = HolhoConverter.FFMPEG_PROCESS_COUNT++;
	}

	public Process getProcess()
	{
		return process;
	}

	private HashMap<OS, String[]> initializeExtMap()
	{
		HashMap<OS, String[]> ext = new HashMap<OS, String[]>();
		ext.put(OS.windows, new String[] { "win", ".exe" });
		ext.put(OS.macos, new String[] { "mac", "" });

		return ext;
	}

	public String getFFMpegBinPath()
	{
		OS os = DirUtils.getPlatform();

		if (ext_map.containsKey(os))
		{
			String[] exts = ext_map.get(os);

			char f = File.separatorChar;

			File bin = null;

			bin = new File(DirUtils.getWorkingDirectory().getAbsolutePath() + f + "natives" + f + "ffmpeg-" + exts[0] + f + "bin" + f + "ffmpeg" + exts[1]);

			if (!bin.exists() || !checkNatives(bin))
			{
				Logger.log(Logger.INFO, "FFMpeg natives not found! Extracting ...");
				extractNatives();
			}
			
			return bin.getAbsolutePath();
		}

		return null;
	}

	public boolean checkNatives(File bin)
	{
		JarFile jar = null;
		try
		{
			String noCopy = "ffmpeg-" + ext_map.get(DirUtils.getPlatform() == OS.windows ? OS.macos : OS.windows)[0];
			
			File jar_or_dir = new File(FFMpegProcess.class.getProtectionDomain().getCodeSource().getLocation().getPath());
			if (jar_or_dir.isFile() && (jar_or_dir.getName().contains(".zip") || jar_or_dir.getName().contains(".jar") || jar_or_dir.getName().contains(".exe")))
			{
				jar = new JarFile(new File(FFMpegProcess.class.getProtectionDomain().getCodeSource().getLocation().getPath()));		
				Enumeration<JarEntry> enumeration = jar.entries();
				while (enumeration.hasMoreElements())
				{
					JarEntry file = (JarEntry) enumeration.nextElement();
					File f = new File(DirUtils.getWorkingDirectory() + File.separator + file.getName());

					if (f.getAbsolutePath().contains(noCopy) || !f.getAbsolutePath().contains("natives"))
						continue;

					if (file.isDirectory())
					{
						f.mkdir();
						continue;
					}

					if(bin.length() == file.getSize())
					{
						return true;
					}
				}
				
			}
			
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			if(jar != null)
				try
				{
					jar.close();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
		}
		
		return false;
	}

	public void start()
	{
		ProcessBuilder pb = new ProcessBuilder();

		pb.redirectErrorStream(true);

		ArrayList<String> params = new ArrayList<String>();

		if (DirUtils.getPlatform() == OS.macos)
		{
			pb.directory((new File(getFFMpegBinPath()).getParentFile()));
		}

		params.add((DirUtils.getPlatform() == OS.macos ? "./ffmpeg" : "\"" + getFFMpegBinPath() + "\""));

		params.addAll(commands);

		pb.command(params);

		try
		{
			process = pb.start();
			if (!muteLog)
				refreshLogs();
		}
		catch (IOException e)
		{
			String commandLine = "";
			for (String command : params)
			{
				commandLine += command + " ";
			}
			Logger.log(Logger.GRAVE, "Error on starting FFMpeg process! Command line : " + commandLine);
			e.printStackTrace();
		}
	}

	private void refreshLogs()
	{
		FFMpegLogger.logLn("[FFMpegProcess id : " + id + "] Process started!");

		new Thread()
		{
			@Override
			public void run()
			{
				try
				{
					InputStreamReader isr = new InputStreamReader(process.getInputStream());
					BufferedReader br = new BufferedReader(isr);
					String line = null;
					while ((line = br.readLine()) != null)
					{
						FFMpegLogger.logLn(line);
					}
				}
				catch (IOException ioe)
				{
					ioe.printStackTrace();
				}

				FFMpegLogger.logLn("[FFMpegProcess id : " + id + "] Process ended!");
			}
		}.start();
	}

	private void extractNatives()
	{
	
		String noCopy = "ffmpeg-" + ext_map.get(DirUtils.getPlatform() == OS.windows ? OS.macos : OS.windows)[0];

		try
		{
			File jar_or_dir = new File(FFMpegProcess.class.getProtectionDomain().getCodeSource().getLocation().getPath());
			if (jar_or_dir.isFile() && (jar_or_dir.getName().contains(".zip") || jar_or_dir.getName().contains(".jar") || jar_or_dir.getName().contains(".exe")))
			{
				JarFile jar = new JarFile(new File(FFMpegProcess.class.getProtectionDomain().getCodeSource().getLocation().getPath()));
				Enumeration<JarEntry> enumeration = jar.entries();
				while (enumeration.hasMoreElements())
				{
					JarEntry file = (JarEntry) enumeration.nextElement();
					File f = new File(DirUtils.getWorkingDirectory() + File.separator + file.getName());

					if (f.getAbsolutePath().contains(noCopy) || !f.getAbsolutePath().contains("natives"))
						continue;

					if (file.isDirectory())
					{
						f.mkdir();
						continue;
					}

					f.getParentFile().mkdirs();

					InputStream is = jar.getInputStream(file);
					FileOutputStream fos = new FileOutputStream(f);

					BufferedOutputStream bout = new BufferedOutputStream(fos, 1024);
					byte[] data = new byte[1024];
					int i = 0;
					int totalDataRead = 0;

					while ((i = is.read(data, 0, 1024)) >= 0)
					{
						totalDataRead += i;
						bout.write(data, 0, i);
						HolhoConverter.instance.mainFrame.updateProgressBar((int) (totalDataRead * 100 / file.getSize()), -1);
					}
					bout.close();
					fos.close();
					is.close();
					
					File binDir = f.getParentFile();
					if(DirUtils.getPlatform() == OS.macos)
					{
						ProcessBuilder pb = new ProcessBuilder();
						pb.directory(binDir);
						pb.command("chmod", "755", "ffmpeg");
						pb.start();
					}
				
				}
				jar.close();
				
			}
			else
			{
				ArrayList<File> onlyFiles = Utils.getOnlyFiles(jar_or_dir.listFiles());

				for (File f : onlyFiles)
				{
					if (!f.getAbsolutePath().contains(noCopy) && f.getAbsolutePath().contains("natives"))
					{
						File srcDir = f.getParentFile();
						while (!srcDir.getName().equals("natives"))
							srcDir = srcDir.getParentFile();

						srcDir = srcDir.getParentFile();

						String toRemove = srcDir.getAbsolutePath().replace("\\", "/");

						String newPath = DirUtils.getWorkingDirectory() + f.getAbsolutePath().replace("\\", "/").replace(toRemove, "").replace("/", "\\");

						Utils.copyFile(f, new File(newPath));
					}
				}
			}
		}
		catch (IOException e)
		{
			HolhoConverter.instance.showPopupMessage(JOptionPane.ERROR_MESSAGE, LangManager.getTranslated("natives.error"), LangManager.getTranslated("popup.title.error"));
			Logger.log(Logger.SEVERE, "Error during natives extraction! Program will exit!");
			e.printStackTrace();
			HolhoConverter.instance.exit();
		}
	}

	public InputStream getProcessInputStream()
	{
		if (process != null)
			return process.getInputStream();

		return null;
	}

	public void setCommands(String... commands)
	{
		for (String command : commands)
		{
			this.commands.add(command);
		}
	}

	public void setCommands(ArrayList<String> commands)
	{
		this.commands.clear();

		for (String command : commands)
		{
			this.commands.add(command);
		}
	}

	public void addCommand(String command)
	{
		this.commands.add(command);
	}

	public void addCommands(ArrayList<String> commands)
	{
		this.commands.addAll(commands);
	}
}
