package it.holhoconverter.converter;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.language.LangManager;
import it.holhoconverter.utils.DirUtils;
import it.holhoconverter.utils.DirUtils.OS;
import it.holhoconverter.utils.Logger;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

public class VideoConverter
{
	public static boolean converting = false;

	public static ArrayList<ConversionPack> conversionpacks = new ArrayList<ConversionPack>();

	private static FFMpegProcess process;

	protected static double conversionDuration;
	protected static double conversionCurrent = 0.0D;

	public static void removePack(int index)
	{
		ConversionPack current = conversionpacks.get(index);
		if (current != null)
		{
			if (!current.converting)
				conversionpacks.remove(index);
			else
				current.toRemove = true;
		}
	}

	public static void invertPacks(int index_0, int index_1)
	{
		ArrayList<ConversionPack> new_list = new ArrayList<ConversionPack>();

		for (int i = 0; i < conversionpacks.size(); i++)
		{
			if (i == index_0 || i == index_1)
				new_list.add(conversionpacks.get(i == index_0 ? index_1 : index_0));
			else
				new_list.add(conversionpacks.get(i));
		}
	}

	public static ConversionPack getNextPack(int index)
	{
		if (index >= conversionpacks.size())
		{
			return null;
		}
		else if (!conversionpacks.get(index).converted)
		{
			return conversionpacks.get(index);
		}
		else
		{
			return getNextPack(++index);
		}
	}

	public static void addToConversionList(ConversionPack pack)
	{
		conversionpacks.add(pack);
	}

	public static void startConversion()
	{
		new Thread()
		{
			@Override
			public void run()
			{
				VideoConverter.start();
			}
		}.start();
	}

	private static double getVideoDuration(VideoInput input)
	{
		final ProcessBuilder pb = new ProcessBuilder();

		FFMpegProcess process = new FFMpegProcess();

		ArrayList<String> params = new ArrayList<String>();

		if (DirUtils.getPlatform() == OS.macos)
		{
			pb.directory((new File(process.getFFMpegBinPath()).getParentFile()));
		}

		params.add((DirUtils.getPlatform() == OS.macos ? "./ffmpeg" : "\"" + process.getFFMpegBinPath() + "\""));

		for (String param : input.getFFMpegCommands())
		{
			params.add(param);
		}

		pb.command(params);

		pb.redirectErrorStream(true);

		try
		{
			Process searchProcess = pb.start();

			if (searchProcess.getInputStream() != null)
			{
				double totalSecs = -1.0D;

				BufferedReader reader = new BufferedReader(new InputStreamReader(searchProcess.getInputStream()));

				String line;

				Scanner sc;

				while ((line = reader.readLine()) != null)
				{
					if (line.contains("Duration:"))
					{
						sc = new Scanner(line);

						Pattern durPattern = Pattern.compile("(?<=Duration: )[^,]*");

						String dur = sc.findWithinHorizon(durPattern, 0);
						if (dur == null)
						{
							sc.close();
							continue;
						}

						String[] hms = dur.split(":");
						if (hms.length == 3)
						{
							totalSecs = Integer.parseInt(hms[0]) * 3600 + Integer.parseInt(hms[1]) * 60 + Double.parseDouble(hms[2]);
							sc.close();
							break;
						}
					}
				}

				reader.close();

				return totalSecs;
			}
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return -1.0D;
	}

	private static double getTotalPacksDuration()
	{
		double total = 0.0D;

		for (int i = 0; i < conversionpacks.size(); i++)
		{
			ConversionPack pack = conversionpacks.get(i);

			if (pack != null && !pack.converted)
			{
				double durCurr = getVideoDuration(pack.getInput());

				if (durCurr != -1.0D)
				{
					total += durCurr;
				}
				else
				{
					Logger.log(Logger.GRAVE, "Couldn't calcutate total duration!");
					return -1.0D;
				}
			}
		}

		return total;
	}

	private static void start()
	{
		converting = true;
		HolhoConverter.instance.mainFrame.updateLanguage();
		HolhoConverter.instance.mainFrame.loadIcons();
		boolean first = true;
		conversionDuration = getTotalPacksDuration();
		conversionCurrent = 0.0D;
		ConversionPack next;
		main: while (converting && (next = getNextPack(0)) != null)
		{
			next.converting = true;

			first = false;

			Logger.log(Logger.INFO, "Processing pack - Input : " + next.getInput().file.getAbsolutePath() + " - Output : " + next.getOutput().file.getAbsolutePath() + " - Format : " + next.getOutput().getFormat().name + " - Conversion Mode : " + next.getConversionMode());

			final ConversionPack current = next;
			if (current.getInput().file != null && !current.getInput().file.exists())
			{
				throwsError(current, "Input file doesn't exist!");
				next.converting = false;
				continue;
			}

			current.getOutput().file.getParentFile().mkdirs();

			final double totalSecs = getVideoDuration(current.getInput());

			
			process = new FFMpegProcess();
			process.setCommands(current.getFFMpegCommands());
			process.start();
			
			if (process.getProcessInputStream() != null)
			{
				
				HolhoConverter.instance.mainFrame.conversionTable.setValueAt(-2, conversionpacks.indexOf(current), 3);

				new Thread()
				{

					@Override
					public void run()
					{
						try
						{
							InputStreamReader isr = new InputStreamReader(process.getProcess().getInputStream());
							BufferedReader br = new BufferedReader(isr);
							String line = null;
							Scanner sc;

							double currentSecs = 0.0D;

							while ((line = br.readLine()) != null && totalSecs != -1.0D)
							{
								if (line.contains("time="))
								{
									sc = new Scanner(line);
									Pattern timePattern = Pattern.compile("(?<=time=)[^ ]*");
									String match = sc.findWithinHorizon(timePattern, 0);

									String[] hms_0 = match.split(":");
									if (hms_0.length == 3)
									{
										if (!current.toRemove)
										{
											conversionCurrent -= currentSecs;
											currentSecs = Integer.parseInt(hms_0[0]) * 3600 + Integer.parseInt(hms_0[1]) * 60 + Double.parseDouble(hms_0[2]);
											conversionCurrent += currentSecs;
											HolhoConverter.instance.mainFrame.updateProgressBar((int) conversionCurrent, (int) conversionDuration);
											double progress = currentSecs / totalSecs;
											current.percent = progress * 100;
											HolhoConverter.instance.mainFrame.conversionTable.setValueAt((int) current.percent, conversionpacks.indexOf(current), 3);

										}
									}
									else
										throwsError(current, "Could not parse current time! (finded=" + match + ")");
								}
							}
						}
						catch (IOException ioe)
						{
							ioe.printStackTrace();
						}
					}
				}.start();

			}
			else
			{
				throwsError(current, "Process was null!");
				current.converting = false;
				continue;
			}

			try
			{
				InputStreamReader isr = new InputStreamReader(process.getProcessInputStream());
				BufferedReader br = new BufferedReader(isr);
				while (br.readLine() != null)
				{
					if (next.toRemove || !converting)
					{
						process.getProcess().destroy();

						if (next.getOutput().file.exists())
						{
							next.getOutput().file.delete();
						}

						if (next.toRemove)
						{
							conversionpacks.remove(next);
							current.converting = false;
							continue main;
						}

						if (!converting)
						{
							next.percent = 0.0D;
							HolhoConverter.instance.mainFrame.conversionTable.setValueAt((int) current.percent, conversionpacks.indexOf(current), 3);
							current.converting = false;
							break main;
						}

					}

				}
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}

			current.converted = true;

			current.converting = false;

			if (current.percent != -1.0D)
			{
				HolhoConverter.instance.mainFrame.conversionTable.setValueAt(101, conversionpacks.indexOf(current), 3);

			}
		}

		if (first)
			HolhoConverter.instance.showPopupMessage(JOptionPane.INFORMATION_MESSAGE, LangManager.getTranslated("conversion.noItem"), LangManager.getTranslated("popup.title.info"));

		converting = false;

		HolhoConverter.instance.mainFrame.updateLanguage();
		HolhoConverter.instance.mainFrame.loadIcons();
	}

	public static void breakProcess()
	{
		if (process != null && process.getProcess() != null)
			process.getProcess().destroy();
	}

	private static void throwsError(ConversionPack current, String error)
	{
		Logger.log(Logger.GRAVE, "Error on processing pack ! " + error);
		current.percent = -1.0D;
		current.converted = true;
		HolhoConverter.instance.mainFrame.conversionTable.setValueAt(-1, conversionpacks.indexOf(current), 3);

	}
}
