package it.holhoconverter.frames;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.Resources;
import it.holhoconverter.converter.ConversionPack;
import it.holhoconverter.converter.FFMpegFormats;
import it.holhoconverter.converter.VideoConverter;
import it.holhoconverter.language.LangManager;
import it.holhoconverter.utils.Utils;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

public class ConversionTable extends JTable
{
	private static final long serialVersionUID = 1L;

	private PopUp popup = new PopUp();

	public HashMap<Integer, ProgressBarCell> progresses = new HashMap<Integer, ProgressBarCell>();

	public ConversionTable()
	{
		this.setModel(new ConversionTableModel());
		this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		registerListeners();
		getTableHeader().setReorderingAllowed(false);
	}

	public int getFirstBlankRow()
	{
		int rows = this.getRowCount();

		for (int i = 0; i < rows; i++)
		{
			if (this.getValueAt(i, 0) == null)
				return i;
		}

		return -1;
	}

	@Override
	public Component prepareRenderer(TableCellRenderer renderer, int row, int column)
	{
		Component comp = super.prepareRenderer(renderer, row, column);

		if (column == 3)
		{
			Object obj = getValueAt(row, column);
			if (obj != null)
			{
				if (!progresses.containsKey(row))
				{
					comp = new ProgressBarCell(this, row, column);
					((ProgressBarCell) comp).setStringPainted(true);
					progresses.put(row, (ProgressBarCell) comp);
				}
				else
				{
					comp = progresses.get(row);
				}
			}
		}

		return comp;
	}

	private void registerListeners()
	{
		this.addMouseListener(new MouseAdapter()
		{
			public void mouseReleased(MouseEvent evnt)
			{
				if (SwingUtilities.isRightMouseButton(evnt))
				{
					Point p = evnt.getPoint();

					int row = rowAtPoint(p);

					if (getValueAt(row, 0) == null)
					{
						return;
					}

					clearSelection();
					setRowSelectionInterval(row, row);

					popup.show(row, evnt.getComponent(), evnt.getX(), evnt.getY());
				}
			}

			public void mouseClicked(MouseEvent evnt)
			{
				if (SwingUtilities.isLeftMouseButton(evnt) && evnt.getClickCount() == 2)
				{
					int row = getSelectedRow();
					int column = getSelectedColumn();

					if (row != -1 && column != -1 && getValueAt(row, column) != null)
					{
						boolean processing = Integer.valueOf(String.valueOf(getModel().getValueAt(row, 3)))  == 0 ? false : true;

						if (!processing && (column == 0 || column == 1))
						{
							int type = column == 0 ? JFileChooser.OPEN_DIALOG : JFileChooser.SAVE_DIALOG;

							File f = showFileChooseDialog(type);

							if (f != null)
							{
								getModel().setValueAt(f.getAbsolutePath(), row, column);
								if(column == 0)
									VideoConverter.conversionpacks.get(row).getInput().file = f;
								else
								{
									FFMpegFormats format = FFMpegFormats.getFormat((String)getValueAt(row, 2));
									VideoConverter.conversionpacks.get(row).getOutput().file = f;
									VideoConverter.conversionpacks.get(row).getOutput().setFormat(format);					
								}		    
							}
							
						}
						else if (!processing && column == 2)
						{
							String message = LangManager.getTranslated("mainframe.table.selectFormat");
							JComboBox<String> formats = new JComboBox<String>();
							formats.setModel(new DefaultComboBoxModel<String>(FFMpegFormats.getNames()));

							int result = JOptionPane.showConfirmDialog(null, new Object[] { message, formats }, LangManager.getTranslated("popup.title.setting"), JOptionPane.OK_CANCEL_OPTION);
							if (result == JOptionPane.OK_OPTION)
							{
								FFMpegFormats new_format = FFMpegFormats.getFormat((String) formats.getSelectedItem());
								getModel().setValueAt(new_format.name, row, column);
								VideoConverter.conversionpacks.get(row).getOutput().setFormat(new_format);
							}

						}
					}
				}
			}
		});
	}

	@Override
	public void setValueAt(Object obj, int row, int column)
	{
		super.setValueAt(obj, row, column);

		Set<Integer> keyset = progresses.keySet();

		for (int key : keyset)
		{
			progresses.get(key).onTableValueChanged();
		}

		HolhoConverter.instance.mainFrame.convTableChanged();
	}

	public void removeSelectedRows()
	{
		int[] rows = getSelectedRows();

		Arrays.sort(rows);

		for (int i = rows.length - 1; i >= 0; i--)
		{
			removeRow(rows[i]);
		}
	}

	public void moveSelectedRows(boolean up)
	{
		int[] rows = getSelectedRows();

		Arrays.sort(rows);

		ArrayList<Integer> cleaned = new ArrayList<Integer>();

		for (int i = 0; i < rows.length; i++)
		{
			if (getValueAt(rows[i], 0) != null)
				cleaned.add(rows[i]);
		}

		rows = new int[cleaned.size()];
		for (int i = 0; i < rows.length; i++)
		{
			rows[i] = cleaned.get(i);
		}

		if (rows.length > 0 && !up ? rows[rows.length - 1] < getFirstBlankRow() - 1 : rows[0] > 0)
		{
			for (int i = up ? 0 : rows.length - 1; up ? i < rows.length : i >= 0; i = up ? i + 1 : i - 1)
			{
				moveRow(rows[i], up);
			}
		}
	}

	public void moveRow(int row, boolean up)
	{
		if (up ? row > 0 : row < HolhoConverter.TABLE_CAPACITY)
		{
			int nextRow = up ? row - 1 : row + 1;
			if (getValueAt(nextRow, 0) != null)
			{
				VideoConverter.invertPacks(row, nextRow);

				invertRows(row, nextRow);
			}
		}
	}

	private void invertRows(int row, int nextRow)
	{
		ProgressBarCell first = progresses.get(row);

		ProgressBarCell newest_first = new ProgressBarCell(first.table, nextRow, first.column);

		ProgressBarCell second = progresses.get(nextRow);

		ProgressBarCell newest_second = new ProgressBarCell(second.table, row, first.column);

		progresses.put(nextRow, newest_first);

		progresses.put(row, newest_second);

		Object[] temp_0 = new Object[4];
		temp_0 = new Object[] { getValueAt(row, 0), getValueAt(row, 1), getValueAt(row, 2), getValueAt(row, 3) };
		Object[] temp_1 = new Object[4];
		temp_1 = new Object[] { getValueAt(nextRow, 0), getValueAt(nextRow, 1), getValueAt(nextRow, 2), getValueAt(nextRow, 3) };

		for (int i = 0; i < temp_0.length; i++)
		{
			setValueAt(temp_0[i], nextRow, i);
		}

		for (int i = 0; i < temp_0.length; i++)
		{
			setValueAt(temp_1[i], row, i);
		}

		this.getSelectionModel().removeSelectionInterval(row, row);
		this.getSelectionModel().addSelectionInterval(nextRow, nextRow);

	}

	public void removeRow(int row)
	{
		ArrayList<ConversionPack> packs = VideoConverter.conversionpacks;

		if (row < packs.size())
		{
			VideoConverter.removePack(row);
		}
		else
		{
			return;
		}

		this.progresses.remove(row);
		((ConversionTableModel) this.getModel()).removeRow(row);
		((ConversionTableModel) this.getModel()).addRow(new Object[] { null, null, null, null });

		HolhoConverter.instance.mainFrame.convTableChanged();
	}

	private File showFileChooseDialog(int type)
	{
		JFileChooser fc = new JFileChooser();
		fc.setMultiSelectionEnabled(false);

		if (type == JFileChooser.OPEN_DIALOG)
			fc.showOpenDialog(null);
		else if (type == JFileChooser.SAVE_DIALOG)
			fc.showSaveDialog(null);

		File f = fc.getSelectedFile();

		if (f != null)
		{
			return f;
		}

		return null;
	}
}

class ConversionTableModel extends DefaultTableModel implements ILangComponent
{
	private static final long serialVersionUID = 1L;

	private String[] titles = { "mainframe.table.input", "mainframe.table.output", "mainframe.table.format", "mainframe.table.progress" };

	public ConversionTableModel()
	{
		super();
		HolhoConverter.instance.registerLangComponent(this);
		loadIdentifiers();
		fillTableBlank();
	}

	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}

	@Override
	public void updateLanguage()
	{
		loadIdentifiers();
	}

	private void loadIdentifiers()
	{
		String[] titles_values = new String[titles.length];

		for (int i = 0; i < titles.length; i++)
		{
			titles_values[i] = LangManager.getTranslated(titles[i]);
		}

		this.setColumnIdentifiers(titles_values);
	}

	private void fillTableBlank()
	{
		Object[] objects = { null, null, null, null };

		for (int i = 0; i < HolhoConverter.TABLE_CAPACITY; i++)
		{
			this.addRow(objects);
		}
	}
}

class PopUp extends JPopupMenu implements ILangComponent
{
	private static final long serialVersionUID = 1L;

	LangMenuItem[] items;
	int row;

	public PopUp()
	{
		items = new LangMenuItem[7];

		items[0] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				ConversionTable tb = HolhoConverter.instance.mainFrame.conversionTable;
				tb.moveRow(tb.getSelectedRow(), true);

			}
		}, "mainframe.moveup", "arrow_up.png");
		items[1] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				ConversionTable tb = HolhoConverter.instance.mainFrame.conversionTable;
				tb.moveRow(tb.getSelectedRow(), false);
			}
		}, "mainframe.movedown", "arrow_down.png");
		items[2] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				ConversionTable tb = HolhoConverter.instance.mainFrame.conversionTable;
				tb.removeRow(tb.getSelectedRow());
			}
		}, "mainframe.removeBtn", "cancel.png");

		items[3] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				int row = HolhoConverter.instance.mainFrame.conversionTable.getSelectedRow();

				ConversionPack pack = VideoConverter.conversionpacks.get(row);

				if (!pack.converting && pack.converted)
				{
					if (pack.getOutput().file.exists())
						Utils.openDirOrFile(pack.getOutput().file.getAbsolutePath());
					else
					{
						HolhoConverter.instance.showPopupMessage(JOptionPane.WARNING_MESSAGE, LangManager.getTranslated("mainframe.table.popup.open.notExists"), LangManager.getTranslated("popup.title.warning"));
						pack.converted = false;
						pack.percent = 0.0D;
						HolhoConverter.instance.mainFrame.conversionTable.setValueAt(0, row, 3);
					}
				}
			}
		}, "mainframe.table.popup.open", "film.png");

		items[4] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				int row = HolhoConverter.instance.mainFrame.conversionTable.getSelectedRow();

				ConversionPack pack = VideoConverter.conversionpacks.get(row);

				if (!pack.getOutput().file.getParentFile().exists())
					pack.getOutput().file.getParentFile().mkdirs();
				Utils.openDirOrFile(pack.getOutput().file.getParentFile().getAbsolutePath());

			}
		}, "mainframe.table.popup.openDir", "folder_camera.png");

		items[5] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				int row = HolhoConverter.instance.mainFrame.conversionTable.getSelectedRow();

				if (!VideoConverter.conversionpacks.get(row).converting)
				{
					ConversionPack pack = VideoConverter.conversionpacks.get(row);
					HolhoConverter.instance.showOptions(pack, row);
				}
			}
		}, "mainframe.table.popup.settings", "cog.png");

		items[6] = initItem(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				int row = HolhoConverter.instance.mainFrame.conversionTable.getSelectedRow();

				if (!VideoConverter.conversionpacks.get(row).converting)
				{
					ConversionPack pack = VideoConverter.conversionpacks.get(row);
					pack.converted = false;
					pack.percent = 0.0D;
					HolhoConverter.instance.mainFrame.conversionTable.setValueAt(0, row, 3);
					return;
				}

				HolhoConverter.instance.showPopupMessage(JOptionPane.INFORMATION_MESSAGE, LangManager.getTranslated("mainframe.table.popup.reset.converting"), LangManager.getTranslated("popup.title.warning"));
			}
		}, "mainframe.table.popup.reset", "arrow_rotate_clockwise.png");

		this.updateLanguage();

		HolhoConverter.instance.registerLangComponent(this);

		for (int i = 0; i < items.length; i++)
		{
			if (i == 3 || i == 5)
				this.add(new JSeparator());

	 		this.add(items[i]);
		}

	}

	public LangMenuItem initItem(ActionListener ap, String langKey, String icon)
	{
		LangMenuItem menuItem = new LangMenuItem(langKey);
		menuItem.addActionListener(ap);
		if (icon != null)
			menuItem.setIcon(Resources.getIcon(HolhoConverter.ICONS_PKG + icon));

		return menuItem;
	}

	public void show(int row, Component invoker, int x, int y)
	{
		super.show(invoker, x, y);
		this.row = row;
		
		ConversionPack pack = VideoConverter.conversionpacks.get(row);
		
		this.items[5].setEnabled(!pack.converting);
		this.items[6].setEnabled(!pack.converting);
		this.items[3].setEnabled(pack.converted);
	}

	@Override
	public void updateLanguage()
	{
		for (int i = 0; i < items.length; i++)
		{
			items[i].updateText();
		}
	}

}

class LangMenuItem extends JMenuItem
{
	private static final long serialVersionUID = 1L;

	public String langKey;

	public LangMenuItem(String langKey)
	{
		this.langKey = langKey;
	}

	public void updateText()
	{
		this.setText(LangManager.getTranslated(langKey));
	}
}