package it.holhoconverter.frames;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.Resources;
import it.holhoconverter.Settings;
import it.holhoconverter.language.LangManager;
import it.holhoconverter.language.LangPack;
import it.holhoconverter.utils.DirUtils;
import it.holhoconverter.utils.Utils;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;

public class HolhoMenu extends JMenuBar
{
	private static final long serialVersionUID = 1L;

	private LangMenu manage;
	private LangMenu info;

	private LangItem openSoftDir;
	private LangItem openVidDir;
	private LangItem close;
	private LangMenu language;

	private LangItem website;
	private LangItem news;
	private LangItem checkUpdates;

	public HolhoMenu()
	{
		manage = new LangMenu("mainframe.menu.manage", "application_edit.png");
		info = new LangMenu("mainframe.menu.help", "help.png");

		openSoftDir = new LangItem("mainframe.menu.manage.openSoftDir", "folder.png");
		openVidDir = new LangItem("mainframe.menu.manage.openVidDir", "folder_camera.png");
		language = new LangMenu("mainframe.menu.manage.language", "world.png");
		close = new LangItem("mainframe.menu.manage.exit", "door_out.png");

		manage.add(openSoftDir);
		manage.add(openVidDir);
		manage.add(language);
		manage.add(new JSeparator());
		manage.add(close);

		checkUpdates = new LangItem("update.try", "arrow_refresh.png");
		website = new LangItem("mainframe.menu.help.website", "world_link.png");
		news = new LangItem("mainframe.menu.help.news", "feed.png");

		if(HolhoConverter.instance.updateMessage)
			info.add(checkUpdates);
		
		info.add(website);
		info.add(news);

		loadLanguages();

		initListeners();

		add(manage);
		add(info);
	}

	private void loadLanguages()
	{
		HashMap<String, LangPack> languages = HolhoConverter.instance.langManager.getLanguages();

		Set<String> keySet = languages.keySet();

		for (String s : keySet)
		{
			LangPack current = languages.get(s);

			JCheckBoxMenuItem item = new JCheckBoxMenuItem();
			item.setForeground(HolhoConverter.instance.colors.text);
			item.setText(current.langName);

			item.setSelected(HolhoConverter.instance.langManager.current == current);

			item.addActionListener(new ActionListener()
			{
				@Override
				public void actionPerformed(ActionEvent arg0)
				{
					HolhoConverter.instance.langManager.setCurrentPack(((JCheckBoxMenuItem) arg0.getSource()).getText());
					refreshLanguages();
				}
			});

			language.add(item);
		}

	}

	private void refreshLanguages()
	{
		for (JCheckBoxMenuItem item : language.langItems)
		{
			item.setSelected(item.getText() == HolhoConverter.instance.langManager.current.langName);
		}
	}

	private void initListeners()
	{
		close.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				HolhoConverter.instance.exit();
			}
		});

		openSoftDir.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				Utils.openDirOrFile(DirUtils.getWorkingDirectory().getAbsolutePath());
			}
		});

		openVidDir.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				Settings settings = HolhoConverter.instance.settings;
				Utils.openDirOrFile(settings.getValue(HolhoConverter.KEY_SAVE_DIR));
			}
		});

		website.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				Settings settings = HolhoConverter.instance.settings;
				Utils.openWebPage(settings.getInternalSettingsValue("WEBSITE"));
			}
		});

		news.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				Settings settings = HolhoConverter.instance.settings;
				Utils.openWebPage(settings.getInternalSettingsValue("NEWS"));
			}
		});
		
		checkUpdates.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				if(!HolhoConverter.instance.updating)
				{
					new Thread(new Runnable()
					{
						public void run()
						{
							HolhoConverter.instance.updating = true;
							HolhoConverter.instance.checkForUpdates(false);
							HolhoConverter.instance.updating = false;
						}
					}).start();
				}		
			}				
		});
	}
}

class LangItem extends JMenuItem implements ILangComponent
{
	private static final long serialVersionUID = 1L;

	public String langKey;
	public String iconName;

	public LangItem(String langKey, String iconName)
	{
		this.langKey = langKey;
		this.iconName = iconName;

		updateLanguage();

		if (iconName != null)
			this.setIcon(Resources.getIcon(HolhoConverter.ICONS_PKG + iconName));

		HolhoConverter.instance.registerLangComponent(this);

		this.setForeground(HolhoConverter.instance.colors.text);
	}

	public void updateLanguage()
	{
		this.setText(LangManager.getTranslated(langKey));
	}
}

class LangMenu extends JMenu implements ILangComponent
{
	private static final long serialVersionUID = 1L;

	public String langKey;
	public String iconName;

	public ArrayList<JCheckBoxMenuItem> langItems = new ArrayList<JCheckBoxMenuItem>();

	public LangMenu(String langKey, String iconName)
	{
		this.langKey = langKey;
		this.iconName = iconName;

		updateLanguage();

		if (iconName != null)
			this.setIcon(Resources.getIcon(HolhoConverter.ICONS_PKG + iconName));

		HolhoConverter.instance.registerLangComponent(this);

		this.setForeground(HolhoConverter.instance.colors.text);
	}

	public void updateLanguage()
	{
		this.setText(LangManager.getTranslated(langKey));
	}

	@Override
	public JMenuItem add(JMenuItem item)
	{
		if (item instanceof JCheckBoxMenuItem)
			this.langItems.add((JCheckBoxMenuItem) item);

		return super.add(item);
	}

}
