package it.holhoconverter.frames;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.Resources;
import it.holhoconverter.Settings;
import it.holhoconverter.converter.ConversionMode;
import it.holhoconverter.converter.ConversionPack;
import it.holhoconverter.converter.FFMpegFormats;
import it.holhoconverter.converter.FFMpegQuality;
import it.holhoconverter.converter.VideoConverter;
import it.holhoconverter.language.LangManager;
import it.holhoconverter.utils.DirUtils;
import it.holhoconverter.utils.Utils;

import java.awt.Cursor;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.awt.font.TextAttribute;
import java.io.File;
import java.util.Map;

import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;
import javax.swing.SwingConstants;

/**
 * 
 * @author Flood - e-mail: Flood2d@hotmail.it - website: www.planetgeeks.org
 */

public class MainFrame extends HolhoFrame
{
	private static final long serialVersionUID = 1L;

	public MainFrame()
	{
		super();

		initComponents();

		initListeners();

		updateLanguage();

		loadIcons();

		this.setCentered();
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void initComponents()
	{
		tableTitle = new JLabel();
		addSrcBtn = new JButton();
		scrollPanel = new JScrollPane();
		conversionTable = new ConversionTable();
		bottomPanel = new JPanel();
		bottomSeparator = new JSeparator();
		infoLabel = new JLabel();
		conversionProgress = new ProgressBarHide();
		vFromatLabel = new JLabel();
		vFormatCombo = new JComboBox<String>();
		qualityCombo = new JComboBox<String>();
		qualityLabel = new JLabel();
		settingsTitle = new JLabel();
		conversionBtn = new JButton();
		advOptionsBtn = new JButton();
		saveDirField = new JTextField();
		saveDirBtn = new JButton();
		saveDirLabel = new JLabel();
		remSrcBtn = new JButton();
		moveDownBtn = new JButton();
		moveUpBtn = new JButton();
		menuBar = new HolhoMenu();

		setMinimumSize(new java.awt.Dimension(676, 350));

		scrollPanel.setViewportView(conversionTable);

		bottomPanel.setBackground(HolhoConverter.instance.colors.bottomPanel);

		infoLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		vFromatLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		Settings settings = HolhoConverter.instance.settings;

		vFormatCombo.setModel(new DefaultComboBoxModel<String>(FFMpegFormats.getNames()));

		String format = null;

		while (format == null)
		{
			format = settings.getValue(HolhoConverter.KEY_FORMAT);

			if (format == null)
				settings.putValue(HolhoConverter.KEY_FORMAT, FFMpegFormats._MP4.name);
		}

		vFormatCombo.setSelectedItem(format);

		qualityCombo.setModel(new DefaultComboBoxModel<String>(FFMpegQuality.getNames()));

		String quality = null;

		while (quality == null)
		{
			quality = settings.getValue(HolhoConverter.KEY_QUALITY);

			if (quality == null)
				settings.putValue(HolhoConverter.KEY_QUALITY, FFMpegQuality.HIGH.name);
		}

		qualityCombo.setSelectedItem(LangManager.getTranslated(quality));

		qualityLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		saveDirLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		String saveDir = null;

		while (saveDir == null)
		{
			saveDir = settings.getValue(HolhoConverter.KEY_SAVE_DIR);

			if (saveDir == null)
				settings.putValue(HolhoConverter.KEY_SAVE_DIR, DirUtils.getWorkingDirectory() + File.separator + HolhoConverter.DEF_OUTPUT_VIDEODIR);
		}

		saveDirField.setText(saveDir);

		Font font = saveDirField.getFont();

		Map attributes = font.getAttributes();
		attributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);

		saveDirField.setFont(font.deriveFont(attributes));

		saveDirField.setEditable(false);

		String conv_key = null;

		while (conv_key == null)
		{
			conv_key = settings.getValue(HolhoConverter.KEY_CONV_MODE);

			if (conv_key == null)
				settings.putValue(HolhoConverter.KEY_CONV_MODE, ConversionMode.PIRAMID.nameKey);
		}

		conversionMode = ConversionMode.getMode(LangManager.getTranslated(conv_key));

		String active = null;

		while (active == null)
		{
		 	active = settings.getValue(HolhoConverter.KEY_AUDIO);

			if (active == null)
				settings.putValue(HolhoConverter.KEY_AUDIO, "true");
		}
		
		activeAudio = active.equals("true");
		
		String bitr = null;

		while (bitr == null)
		{
			bitr = settings.getValue(HolhoConverter.KEY_BITRATE);

			if (bitr == null)
				settings.putValue(HolhoConverter.KEY_BITRATE, "-1");
		}
		
		bitrate = Integer.valueOf(bitr);	

		conversionMode = ConversionMode.getMode(LangManager.getTranslated(conv_key));

		GroupLayout bottomPanelLayout = new GroupLayout(bottomPanel);
		bottomPanel.setLayout(bottomPanelLayout);
		bottomPanelLayout.setHorizontalGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addComponent(bottomSeparator).addGroup(bottomPanelLayout.createSequentialGroup().addContainerGap().addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup().addComponent(infoLabel, GroupLayout.DEFAULT_SIZE, 183, Short.MAX_VALUE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(conversionProgress, GroupLayout.PREFERRED_SIZE, 459, GroupLayout.PREFERRED_SIZE)).addComponent(settingsTitle, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addGroup(GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup().addGap(0, 0, Short.MAX_VALUE).addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING, false).addGroup(bottomPanelLayout.createSequentialGroup().addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup().addComponent(qualityLabel, GroupLayout.PREFERRED_SIZE, 88, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(qualityCombo, GroupLayout.PREFERRED_SIZE, 160, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(vFromatLabel, GroupLayout.PREFERRED_SIZE, 88, GroupLayout.PREFERRED_SIZE)).addComponent(advOptionsBtn, GroupLayout.Alignment.TRAILING, GroupLayout.PREFERRED_SIZE, 160, GroupLayout.PREFERRED_SIZE)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING, false).addComponent(conversionBtn, GroupLayout.Alignment.TRAILING, GroupLayout.DEFAULT_SIZE, 160, Short.MAX_VALUE).addComponent(vFormatCombo, GroupLayout.Alignment.TRAILING, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))).addGroup(GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup().addComponent(saveDirLabel, GroupLayout.DEFAULT_SIZE, 161, Short.MAX_VALUE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(saveDirField, GroupLayout.PREFERRED_SIZE, 313, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(saveDirBtn, GroupLayout.PREFERRED_SIZE, 28, GroupLayout.PREFERRED_SIZE))))).addContainerGap()));
		bottomPanelLayout.setVerticalGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(GroupLayout.Alignment.TRAILING, bottomPanelLayout.createSequentialGroup().addContainerGap().addComponent(settingsTitle).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(vFromatLabel).addComponent(vFormatCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE).addComponent(qualityCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE).addComponent(qualityLabel)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(saveDirField, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE).addComponent(saveDirBtn).addComponent(saveDirLabel)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(conversionBtn).addComponent(advOptionsBtn)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addGroup(bottomPanelLayout.createParallelGroup(GroupLayout.Alignment.TRAILING, false).addGroup(bottomPanelLayout.createSequentialGroup().addComponent(bottomSeparator, GroupLayout.PREFERRED_SIZE, 5, GroupLayout.PREFERRED_SIZE).addGap(17, 17, 17)).addComponent(conversionProgress, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE).addComponent(infoLabel, GroupLayout.Alignment.LEADING, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))));

		setJMenuBar(menuBar);

		convTableChanged();

		GroupLayout layout = new GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(GroupLayout.Alignment.TRAILING, layout.createSequentialGroup().addContainerGap().addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING).addComponent(scrollPanel).addGroup(layout.createSequentialGroup().addComponent(tableTitle, GroupLayout.DEFAULT_SIZE, 184, Short.MAX_VALUE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(moveUpBtn, GroupLayout.PREFERRED_SIZE, 110, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(moveDownBtn, GroupLayout.PREFERRED_SIZE, 110, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(remSrcBtn, GroupLayout.PREFERRED_SIZE, 110, GroupLayout.PREFERRED_SIZE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(addSrcBtn, GroupLayout.PREFERRED_SIZE, 110, GroupLayout.PREFERRED_SIZE))).addContainerGap()).addComponent(bottomPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE));
		layout.setVerticalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(layout.createSequentialGroup().addContainerGap().addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(addSrcBtn).addComponent(tableTitle).addComponent(remSrcBtn, GroupLayout.PREFERRED_SIZE, 28, GroupLayout.PREFERRED_SIZE).addComponent(moveUpBtn, GroupLayout.PREFERRED_SIZE, 28, GroupLayout.PREFERRED_SIZE).addComponent(moveDownBtn, GroupLayout.PREFERRED_SIZE, 28, GroupLayout.PREFERRED_SIZE)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(scrollPanel, GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addComponent(bottomPanel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)));

		pack();
	}

	private void initListeners()
	{
		this.addSrcBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				JFileChooser fc = new JFileChooser();

				Settings settings = HolhoConverter.instance.settings;

				String last_dir = settings.getValue(HolhoConverter.KEY_LAST_DIR);
				if (last_dir != null)
				{
					File folder = new File(last_dir);
					if (folder.exists() && folder.isDirectory())
						fc.setCurrentDirectory(folder);
				}

				fc.setFileSelectionMode(JFileChooser.FILES_ONLY);

				fc.setMultiSelectionEnabled(true);

				fc.showOpenDialog(null);

				if (fc.getSelectedFiles() != null && fc.getSelectedFiles().length > 0)
				{
					settings.putValue(HolhoConverter.KEY_LAST_DIR, fc.getSelectedFiles()[0].getParentFile().getAbsolutePath());

					File[] files = fc.getSelectedFiles();
					main: for (File f : files)
					{
						String src_path = f.getAbsolutePath();
						FFMpegFormats conv_format = FFMpegFormats.getFormat((String)vFormatCombo.getSelectedItem());
						String dest_path = HolhoConverter.instance.settings.getValue(HolhoConverter.KEY_SAVE_DIR) + File.separator + Utils.replaceFileExtension(f.getName(), conv_format.extension);
						int percent = 0;

						Object[] objects = { src_path, dest_path, conv_format, percent };
						int blankRow = conversionTable.getFirstBlankRow();
						if (blankRow != -1)
						{
							ConversionPack conv_pack = new ConversionPack(src_path, dest_path, conv_format, FFMpegQuality.getQuality((String) qualityCombo.getSelectedItem()), ConversionMode.PIRAMID);
							conv_pack.getOutput().activeAudio = activeAudio;
							conv_pack.getOutput().bitrate = bitrate;
							conv_pack.conv_mode = conversionMode;

							VideoConverter.addToConversionList(conv_pack);

							for (int i = 0; i < objects.length; i++)
							{
								conversionTable.setValueAt(i == 2 ? ((FFMpegFormats) objects[i]).name : objects[i], blankRow, i);
							}
						}
						else
						{
							HolhoConverter.instance.showPopupMessage(JOptionPane.WARNING_MESSAGE, LangManager.getTranslated("mainframe.table.limitReached"), LangManager.getTranslated("popup.title.warning"));
							break main;
						}
					}
				}

			}
		});

		this.addWindowListener(new IWindowListener()
		{
			@Override
			public void windowClosing(WindowEvent arg0)
			{
			
				HolhoConverter.instance.exit();
			}
		});

		this.conversionBtn.addActionListener(new ActionListener()
		{

			@Override
			public void actionPerformed(ActionEvent e)
			{
				if (!VideoConverter.converting)
					VideoConverter.startConversion();
				else
					VideoConverter.converting = false;
			}
		});

		this.remSrcBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				conversionTable.removeSelectedRows();
			}
		});

		this.moveDownBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				conversionTable.moveSelectedRows(false);
			}
		});

		this.moveUpBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				conversionTable.moveSelectedRows(true);
			}
		});

		this.saveDirBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				JFileChooser fc = new JFileChooser();

				fc.setCurrentDirectory(new File(HolhoConverter.instance.settings.getValue(HolhoConverter.KEY_SAVE_DIR)).getParentFile());

				fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

				fc.setMultiSelectionEnabled(false);

				fc.showOpenDialog(null);

				if (fc.getSelectedFile() != null && fc.getSelectedFile().isDirectory())
				{
					saveDirField.setText(fc.getSelectedFile().getAbsolutePath());
					HolhoConverter.instance.settings.putValue(HolhoConverter.KEY_SAVE_DIR, fc.getSelectedFile().getAbsolutePath());

					for (int i = 0; i < VideoConverter.conversionpacks.size(); i++)
					{
						ConversionPack conv = VideoConverter.conversionpacks.get(i);

						if (!conv.converting)
							conv.getOutput().file = new File(HolhoConverter.instance.settings.getValue(HolhoConverter.KEY_SAVE_DIR + File.separator + conv.getOutput().file.getName()));

						conversionTable.setValueAt(conv.getOutput().file.getAbsolutePath(), i, 1);
					}
				}
			}
		});

		this.saveDirField.addMouseListener(new MouseAdapter()
		{

			@Override
			public void mouseClicked(MouseEvent arg0)
			{
				Utils.openDirOrFile(HolhoConverter.instance.settings.getValue(HolhoConverter.KEY_SAVE_DIR));
			}

			@Override
			public void mouseEntered(MouseEvent arg0)
			{
				HolhoConverter.instance.mainFrame.setCursor(new Cursor(Cursor.HAND_CURSOR));
			}

			@Override
			public void mouseExited(MouseEvent arg0)
			{
				HolhoConverter.instance.mainFrame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
		});

		this.qualityCombo.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				FFMpegQuality quality = FFMpegQuality.getQuality((String) qualityCombo.getSelectedItem());

				HolhoConverter.instance.settings.putValue(HolhoConverter.KEY_QUALITY, quality.name);

				for (ConversionPack pack : VideoConverter.conversionpacks)
				{
					pack.getOutput().quality = quality;
				}
			}
		});

		this.vFormatCombo.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				FFMpegFormats format = FFMpegFormats.getFormat((String) vFormatCombo.getSelectedItem());

				HolhoConverter.instance.settings.putValue(HolhoConverter.KEY_FORMAT, format.name);

				for (ConversionPack pack : VideoConverter.conversionpacks)
				{
					pack.getOutput().setFormat(format);
				}

				for (int i = 0; i < conversionTable.getRowCount(); i++)
				{
					if (conversionTable.getValueAt(i, 2) != null)
					{
						conversionTable.setValueAt(format.name, i, 2);
					}
					else
					{
						break;
					}
				}
			}
		});
		
		this.advOptionsBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
                HolhoConverter.instance.showOptions(null, -1);
			}		
		});
	}

	public void loadIcons()
	{
		String prefix = HolhoConverter.ICONS_PKG;

		this.addSrcBtn.setIcon(Resources.getIcon(prefix + "add.png"));

		this.remSrcBtn.setIcon(Resources.getIcon(prefix + "cancel.png"));

		this.moveDownBtn.setIcon(Resources.getIcon(prefix + "arrow_down.png"));

		this.moveUpBtn.setIcon(Resources.getIcon(prefix + "arrow_up.png"));

		this.conversionBtn.setIcon(VideoConverter.converting ? Resources.getIcon(prefix + "control_stop_blue.png") : Resources.getIcon(prefix + "control_play_blue.png"));

		this.advOptionsBtn.setIcon(Resources.getIcon(prefix + "wrench.png"));

		this.saveDirBtn.setIcon(Resources.getIcon(prefix + "folder_explore.png"));
	}

	@Override
	public void updateLanguage()
	{
		this.setTitle(LangManager.getTranslated("mainframe.title") + " " + HolhoConverter.instance.settings.getInternalSettingsValue("VERSION"));

		this.tableTitle.setText(LangManager.getTranslated("mainframe.tabletitle"));

		this.addSrcBtn.setText(LangManager.getTranslated("mainframe.addBtn"));

		this.remSrcBtn.setText(LangManager.getTranslated("mainframe.removeBtn"));

		this.moveDownBtn.setText(LangManager.getTranslated("mainframe.movedown"));

		this.moveUpBtn.setText(LangManager.getTranslated("mainframe.moveup"));

		this.vFromatLabel.setText(LangManager.getTranslated("mainframe.bottompanel.formatLbl"));

		this.saveDirLabel.setText(LangManager.getTranslated("mainframe.bottompanel.saveDirLbl"));

		this.conversionBtn.setText(VideoConverter.converting ? LangManager.getTranslated("mainframe.bottompanel.stopBtn") : LangManager.getTranslated("mainframe.bottompanel.convertBtn"));

		this.qualityLabel.setText(LangManager.getTranslated("mainframe.bottompanel.qualityLbl"));

		this.settingsTitle.setText(LangManager.getTranslated("mainframe.bottompanel.title"));

		this.advOptionsBtn.setText(LangManager.getTranslated("mainframe.bottompanel.advOptionsBtn"));

		this.infoLabel.setText(LangManager.getTranslated("mainframe.bottompanel.infoLbl.totalProgress"));

		int currentFormat = qualityCombo.getSelectedIndex();

		qualityCombo.setModel(new DefaultComboBoxModel<String>(FFMpegQuality.getNames()));

		qualityCombo.setSelectedIndex(currentFormat);

		this.setInfo(VideoConverter.converting, !VideoConverter.converting ? LangManager.getTranslated("mainframe.bottompanel.infoLbl.wait") : LangManager.getTranslated("mainframe.bottompanel.infoLbl.totalProgress"));
	}

	public void convTableChanged()
	{
		this.addSrcBtn.setEnabled(VideoConverter.conversionpacks.size() < HolhoConverter.TABLE_CAPACITY);
		this.remSrcBtn.setEnabled(VideoConverter.conversionpacks.size() > 0);
		this.moveDownBtn.setEnabled(VideoConverter.conversionpacks.size() > 0);
		this.moveUpBtn.setEnabled(VideoConverter.conversionpacks.size() > 0);
	}

	public void setInfo(boolean converting, String message)
	{
		if (converting)
		{
			this.conversionProgress.draw_flag = true;
			this.infoLabel.setHorizontalAlignment(SwingConstants.RIGHT);
			this.conversionProgress.repaint();
		}
		else
		{
			this.conversionProgress.draw_flag = false;
			this.infoLabel.setHorizontalAlignment(SwingConstants.LEFT);
			this.conversionProgress.repaint();
		}

		this.infoLabel.setText(message);
	}

	public String[] getCurrentSettings()
	{
		String[] returned = new String[5];

		returned[0] = (String) qualityCombo.getSelectedItem();
		returned[1] = (String) vFormatCombo.getSelectedItem();
		returned[2] = LangManager.getTranslated(conversionMode.nameKey);
		returned[3] = activeAudio ? "true" : "false";
		returned[4] = String.valueOf(bitrate);

		return returned;
	}
	
	public void updateProgressBar(int conversionCurrent, int conversionDuration)
	{	
		if(conversionDuration == -1)
		{
			conversionProgress.extractNatives = true;
			conversionProgress.setValue(conversionCurrent);
			conversionProgress.setStringPainted(true);
			conversionProgress.setString(LangManager.getTranslated("natives.extracting"));
		}
		else
		{
			conversionProgress.extractNatives = false;
			conversionProgress.setMaximum(100);
			conversionProgress.setValue(100*conversionCurrent/conversionDuration);
		}
	}
	
	public ConversionMode conversionMode;
	public boolean activeAudio;
	public int bitrate;
	private JButton addSrcBtn;
	private JButton advOptionsBtn;
	private JPanel bottomPanel;
	private JSeparator bottomSeparator;
	private JButton conversionBtn;
	private ProgressBarHide conversionProgress;
	public ConversionTable conversionTable;
	private JLabel infoLabel;
	private JScrollPane scrollPanel;
	private HolhoMenu menuBar;
	private JButton moveDownBtn;
	private JButton moveUpBtn;
	public JComboBox<String> qualityCombo;
	private JLabel qualityLabel;
	private JButton remSrcBtn;
	private JButton saveDirBtn;
	private JTextField saveDirField;
	private JLabel saveDirLabel;
	private JLabel settingsTitle;
	private JLabel tableTitle;
	public JComboBox<String> vFormatCombo;
	private JLabel vFromatLabel;
}
