package it.holhoconverter.frames;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.Resources;
import it.holhoconverter.converter.ConversionMode;
import it.holhoconverter.converter.ConversionPack;
import it.holhoconverter.converter.FFMpegFormats;
import it.holhoconverter.converter.FFMpegQuality;
import it.holhoconverter.converter.VideoConverter;
import it.holhoconverter.language.LangManager;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.JToggleButton;
import javax.swing.LayoutStyle;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;

public class OptionsFrame extends HolhoFrame
{

	private static final long serialVersionUID = 1L;
	private int row;

	public OptionsFrame()
	{
		initComponents();

		updateLanguage();

		setCentered();

		loadListeners();

		this.setResizable(false);
	}

	private void initComponents()
	{
		modeLabel = new JLabel();
		modeCombo = new JComboBox<String>();
		qualityLabel = new JLabel();
		qualityCombo = new JComboBox<String>();
		formatLabel = new JLabel();
		formatCombo = new JComboBox<String>();
		bitrateLabel = new JLabel();
		bitrateSpinner = new JSpinner(new SpinnerNumberModel(0.0, 0.0, 999999999.0, 10.0));
		audioCheck = new JCheckBox();
		saveBtn = new JButton();
		discardBtn = new JButton();
		resetBtn = new JToggleButton();

		setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

		modeLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		modeCombo.setModel(new DefaultComboBoxModel<String>(ConversionMode.getNames()));

		qualityLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		qualityCombo.setModel(new DefaultComboBoxModel<String>(FFMpegQuality.getNames()));

		formatLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		formatCombo.setModel(new DefaultComboBoxModel<String>(FFMpegFormats.getNames()));

		bitrateLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		audioCheck.setHorizontalAlignment(SwingConstants.CENTER);

		GroupLayout layout = new GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(layout.createSequentialGroup().addGap(19, 19, 19).addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING, false).addGroup(layout.createSequentialGroup().addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING, false).addComponent(modeLabel, GroupLayout.DEFAULT_SIZE, 98, Short.MAX_VALUE).addComponent(qualityLabel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(formatLabel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(bitrateLabel, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED).addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING, false).addComponent(modeCombo, GroupLayout.Alignment.LEADING, 0, 176, Short.MAX_VALUE).addComponent(qualityCombo, GroupLayout.Alignment.LEADING, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(formatCombo, GroupLayout.Alignment.LEADING, 0, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(bitrateSpinner, GroupLayout.Alignment.LEADING))).addGroup(GroupLayout.Alignment.TRAILING, layout.createSequentialGroup().addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING).addComponent(audioCheck, GroupLayout.Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 132, Short.MAX_VALUE).addComponent(discardBtn, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)).addGap(18, 18, 18).addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING, false).addComponent(saveBtn, GroupLayout.DEFAULT_SIZE, GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE).addComponent(resetBtn, GroupLayout.DEFAULT_SIZE, 130, Short.MAX_VALUE)))).addContainerGap(19, Short.MAX_VALUE)));
		layout.setVerticalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(layout.createSequentialGroup().addGap(24, 24, 24).addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(modeLabel).addComponent(modeCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)).addGap(18, 18, 18).addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(qualityLabel).addComponent(qualityCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)).addGap(18, 18, 18).addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(formatLabel).addComponent(formatCombo, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)).addGap(18, 18, 18).addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(bitrateLabel).addComponent(bitrateSpinner, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)).addGap(18, 18, 18).addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(audioCheck).addComponent(resetBtn)).addPreferredGap(LayoutStyle.ComponentPlacement.RELATED, 38, Short.MAX_VALUE).addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(saveBtn).addComponent(discardBtn)).addGap(16, 16, 16)));

		pack();
	}

	private void loadListeners()
	{
		saveBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{

				int index_start = row != -1 ? row : 0;
				int index_max = row != -1 ? row + 1 : VideoConverter.conversionpacks.size();

				for (int i = index_start; i < index_max; i++)
				{
					ConversionPack pack = VideoConverter.conversionpacks.get(i);

					if (!pack.converting)
					{
						pack.conv_mode = ConversionMode.getMode((String) modeCombo.getSelectedItem());
						
					
						pack.getOutput().bitrate = (Integer.valueOf(String.valueOf(bitrateSpinner.getValue()))) <= 0 ? -1 : (Integer.valueOf(String.valueOf(bitrateSpinner.getValue())));
						pack.getOutput().activeAudio = audioCheck.isSelected();
						pack.getOutput().setFormat(FFMpegFormats.getFormat((String) formatCombo.getSelectedItem()));
						pack.getOutput().quality = FFMpegQuality.getQuality((String) qualityCombo.getSelectedItem());

						if (resetBtn.isSelected())
						{
							pack.converted = false;
							pack.percent = 0.0D;
							HolhoConverter.instance.mainFrame.conversionTable.setValueAt(0, i, 3);
						}

						HolhoConverter.instance.mainFrame.conversionTable.setValueAt(pack.getOutput().getFormat().name, i, 2);

					}
				}

				if (row == -1)
				{
					MainFrame main = HolhoConverter.instance.mainFrame;
					main.activeAudio = audioCheck.isSelected();
					main.bitrate = Integer.valueOf(String.valueOf(bitrateSpinner.getValue()));
					main.conversionMode = ConversionMode.getMode((String) modeCombo.getSelectedItem());
					main.vFormatCombo.setSelectedItem(formatCombo.getSelectedItem());
					main.qualityCombo.setSelectedItem(qualityCombo.getSelectedItem());
				}

				setVisible(false);

			}
		});

		discardBtn.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0)
			{
				setVisible(false);
			}
		});
	}

	public void show(ConversionPack pack, int row)
	{
		if (!this.isVisible())
		{
			updateLanguage();
			
			if (pack != null)
			{
				formatCombo.setSelectedItem(pack.getOutput().getFormat().name);
				qualityCombo.setSelectedItem(LangManager.getTranslated(pack.getOutput().quality.name));
				modeCombo.setSelectedItem(LangManager.getTranslated(pack.getConversionMode().nameKey));
				bitrateSpinner.setValue(pack.getOutput().bitrate <= -1 ? 0 : pack.getOutput().bitrate);
				audioCheck.setSelected(pack.getOutput().activeAudio);

				this.row = row;
			}
			else
			{
				String[] arr = HolhoConverter.instance.mainFrame.getCurrentSettings();

				qualityCombo.setSelectedItem(arr[0]);
				formatCombo.setSelectedItem(arr[1]);
				modeCombo.setSelectedItem(arr[2]);
				audioCheck.setSelected(arr[3] == "true");
				bitrateSpinner.setValue(Integer.valueOf(arr[4]) <= -1 ? 0 : Integer.valueOf(arr[4]));

				this.row = -1;
			}

			
			this.setIconImage(Resources.getIcon(HolhoConverter.ICONS_PKG + (row <= -1 ? "wrench.png" : "cog.png")).getImage());
			this.setVisible(true);
		}
	}

	private JCheckBox audioCheck;
	private JLabel bitrateLabel;
	private JSpinner bitrateSpinner;
	private JButton discardBtn;
	private JComboBox<String> formatCombo;
	private JLabel formatLabel;
	private JComboBox<String> modeCombo;
	private JLabel modeLabel;
	private JComboBox<String> qualityCombo;
	private JLabel qualityLabel;
	private JToggleButton resetBtn;
	private JButton saveBtn;

	@Override
	public void updateLanguage()
	{
		this.setTitle(LangManager.getTranslated("optionsframe.title") + " [" + LangManager.getTranslated(row <= -1 ? "optionsframe.title.all" : "optionsframe.title.pack") + (row <= -1 ? "" : row + 1) + "]");

		String langName = HolhoConverter.instance.langManager.latestPackName;
		
		String key = LangManager.getKey(langName, (String)modeCombo.getSelectedItem());
		
		modeCombo.setModel(new DefaultComboBoxModel<String>(ConversionMode.getNames()));
		
		modeCombo.setSelectedItem(LangManager.getTranslated(key));
		
		key = LangManager.getKey(langName, (String)qualityCombo.getSelectedItem());
		
		qualityCombo.setModel(new DefaultComboBoxModel<String>(FFMpegQuality.getNames()));
		
		qualityCombo.setSelectedItem(LangManager.getTranslated(key));
		
		key = LangManager.getKey(langName, (String)formatCombo.getSelectedItem());
		
		formatCombo.setModel(new DefaultComboBoxModel<String>(FFMpegFormats.getNames()));
		
		formatCombo.setSelectedItem(LangManager.getTranslated(key));
		
		modeLabel.setText(LangManager.getTranslated("optionsframe.mode.label"));

		qualityLabel.setText(LangManager.getTranslated("mainframe.bottompanel.qualityLbl"));

		formatLabel.setText(LangManager.getTranslated("mainframe.bottompanel.formatLbl"));

		bitrateLabel.setText(LangManager.getTranslated("optionsframe.bitrate.label"));

		audioCheck.setText(LangManager.getTranslated("optionsframe.audio.label"));

		saveBtn.setText(LangManager.getTranslated("optionsframe.button.save"));

		discardBtn.setText(LangManager.getTranslated("optionsframe.button.discard"));

		resetBtn.setText(LangManager.getTranslated("optionsframe.button.reset"));

	}
}
