package it.holhoconverter.language;

import it.holhoconverter.HolhoConverter;
import it.holhoconverter.Resources;
import it.holhoconverter.Settings;
import it.holhoconverter.utils.Logger;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Set;

public class LangManager
{
	private LinkedHashMap<String, LangPack> languages = new LinkedHashMap<String, LangPack>();

	public LangPack current;

	public String latestPackName;

	public LangManager()
	{
		loadLanguages();
	}

	public HashMap<String, LangPack> getLanguages()
	{
		return languages;
	}

	public boolean setCurrentPack(String langName)
	{
		if (languages.containsKey(langName))
		{
			latestPackName = current == null ? langName : current.langName;
			current = languages.get(langName);
			HolhoConverter.instance.settings.putValue(HolhoConverter.KEY_LANG, current.langName);
			Logger.log(Logger.INFO, "Setted lang with name : " + langName);
			HolhoConverter.instance.updateLanguages();
			return true;
		}

		return false;
	}

	public static String getTranslated(String key)
	{
		LangManager langManager = HolhoConverter.instance.langManager;

		if (langManager.current != null)
			return langManager.current.getTranslated(key);

		return key;
	}

	public static String getKey(String langName, String translated)
	{
		LangManager langManager = HolhoConverter.instance.langManager;

		if (langManager.languages.containsKey(langName))
			return langManager.languages.get(langName).getKey(translated);

		return translated;
	}

	public static String getKey(String translated)
	{
		LangManager langManager = HolhoConverter.instance.langManager;

		if (langManager.current != null)
			return langManager.current.getKey(translated);

		return translated;
	}

	private void loadLanguages()
	{
		InputStream input = Resources.getInputStream(HolhoConverter.LANGS_PKG + HolhoConverter.LANGS_LIST);

		try
		{
			if (input != null)
			{
				BufferedReader br = new BufferedReader(new InputStreamReader(input));

				String line;

				while ((line = br.readLine()) != null)
				{
					if (line.contains("="))
					{
						String langName = line.split("=")[0];
						String fileName = line.substring(langName.length() + 1);

						InputStream is = Resources.getInputStream(HolhoConverter.LANGS_PKG + fileName);

						if (is != null)
						{
							LangPack pack = new LangPack(langName, is);
							if (pack != null)
								languages.put(langName, pack);
						}
						else
							Logger.log(Logger.GRAVE, "Error on loading lang with name : " + langName + "! InputStream was null!");
					}
				}
				br.close();
			}
			else
			{
				Logger.log(Logger.GRAVE, "Error on loading language pack ! InputStream param was null!");
			}
		}
		catch (IOException e)
		{
			Logger.log(Logger.GRAVE, "Error on loading language pack!");
			e.printStackTrace();
		}

		Settings settings = HolhoConverter.instance.settings;
		String lastLang = settings.getValue(HolhoConverter.KEY_LANG);

		boolean setFirst = false;

		if (lastLang != null)
		{
			if (!setCurrentPack(lastLang))
			{
				setFirst = true;
			}
		}
		else
		{
			setFirst = true;
		}

		if (setFirst)
		{
			Set<String> keys = languages.keySet();

			for (String key : keys)
			{
				setCurrentPack(key);
				settings.putValue(HolhoConverter.KEY_LANG, key);
				break;
			}
		}
	}
}
