package it.holhoconverter.utils;

import it.holhoconverter.HolhoConverter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.regex.Pattern;


public enum Logger
{	
	INFO("[INFO]"), 
	GRAVE("[GRAVE]"), 
	SEVERE("[SEVERE]");

	String type;

	Logger(String type)
	{
		this.type = type;
	}

	private static boolean initialized = false;
	
	private static File logFolder = new File(DirUtils.getWorkingDirectory() + File.separator + HolhoConverter.LOG_FOLDER);
	
	private static File currentLog;
	
	private static String getDate(String regex)
	{
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat(regex);
		return sdf.format(cal.getTime());
	}

	public static void log(Logger log, String str)
	{
		if(initialized)
		{
			System.out.println("[" + getDate("HH:mm:ss") + "]" + "[HolhoConverter]" + log.type + " : " + str);
		}
	}

	private static File getLogFolder()
	{
		if (!logFolder.exists())
		{
			logFolder.mkdirs();
		}
		return logFolder;
	}
	
	public static void loadLogger()
	{
		if(!initialized)
		{
			initialized = true;
			
			try
			{
				currentLog = new File(getLogFolder() + File.separator + HolhoConverter.LOG_FILE_PREFIX + getDate("yyyy.MM.dd.HH.mm.ss"));
				
				File[] logs = getLogFolder().listFiles();
				
				int logsMax = HolhoConverter.MAX_LOGS;
				
				int logsLenght = logs.length;
				
				if (logs != null && logsLenght >= logsMax)
				{
					for(int i = 0; i < logsLenght - (logsMax - 1); i++)
					{
						deleteOldestLog(logs);
						logs = getLogFolder().listFiles();
					}
				}
				
				FileOutputStream file = new FileOutputStream(currentLog.getAbsolutePath());
				LogPrintStream lpr = new LogPrintStream(file, System.out);
				System.setOut(lpr);
				System.setErr(lpr);
			}
			catch (FileNotFoundException e)
			{
				System.out.println("Error on loading HoloPlayer Logger!");
				e.printStackTrace();
			}
		}
	}
	
	private static void deleteOldestLog(File[] logs)
	{
		File toDelete = null;
		for (int a = 0; a < 6; a++)
		{
			boolean breaked = false;
			for (int i = 0; i < logs.length; i++)
			{
				String[] date = logs[i].getName().substring(HolhoConverter.LOG_FILE_PREFIX.length()).split(Pattern.quote("."));
				if (date.length != 6)
				{
					toDelete = logs[i];
					break;
				}
				int first = 0;
				try
				{
					first = Integer.valueOf(date[a]);
				}
				catch (NumberFormatException e)
				{
					toDelete = logs[i];
					break;
				}
				if (toDelete != null)
				{
					String[] current = logs[i].getName().substring(4).split(Pattern.quote("."));

					if (first < Integer.valueOf(current[a]))
					{
						toDelete = logs[i];
					}
				}
				else
				{
					toDelete = logs[i];
				}
			}
			if (breaked)
			{
				break;
			}
		}
		toDelete.delete();
	}
}
