package it.holhoconverter.utils;

import java.awt.Desktop;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.regex.Pattern;

public class Utils
{
	public static ArrayList<String> arrayToList(String[] arr)
	{
		ArrayList<String> list = new ArrayList<String>();

		if (arr != null)
			for (String s : arr)
				list.add(s);

		return list;
	}

	public static String[] listToArray(ArrayList<String> list)
	{
		String[] s = new String[list.size()];

		for (int i = 0; i < s.length; i++)
		{
			s[i] = list.get(i);
		}

		return s;
	}

	public static String replaceFileExtension(String initial, String newExtension)
	{
		if (!initial.contains("."))
			return initial + "." + newExtension;

		String[] spt = initial.split(Pattern.quote("."));

		return initial.substring(0, initial.length() - spt[spt.length - 1].length()) + newExtension;
	}

	public static void openDirOrFile(String path)
	{
		try
		{
			Desktop.getDesktop().open(new File(path));
		}
		catch (IOException e)
		{
			Logger.log(Logger.GRAVE, "Error on opening file : " + path);
			e.printStackTrace();
		}
	}

	public static void openWebPage(String url)
	{
		try
		{
			Desktop.getDesktop().browse(new URI(url));
		}
		catch (IOException e)
		{
			Logger.log(Logger.GRAVE, "Error on opening webpage '" + url + "'!");
		}
		catch (URISyntaxException e)
		{
			Logger.log(Logger.GRAVE, "Error on opening webpage '" + url + "'! Failed to read url sintax!");
		}
	}

	public static ArrayList<File> getOnlyFiles(File... arr)
	{
		ArrayList<File> onlyFiles = new ArrayList<File>();

		for (File f : arr)
		{
			if (f.isDirectory())
				onlyFiles.addAll(getOnlyFiles(f.listFiles()));
			else
				onlyFiles.add(f);
		}

		return onlyFiles;
	}

	public static void copyFile(File src, File dest)
	{
		try
		{
			File inputFile = new File(src.getAbsolutePath());
			File outputFile = new File(dest.getAbsolutePath());
			if (!outputFile.getParentFile().exists())
				outputFile.getParentFile().mkdirs();
			InputStream finput = new BufferedInputStream(new FileInputStream(inputFile));
			OutputStream foutput = new BufferedOutputStream(new FileOutputStream(outputFile));
			byte[] buffer = new byte[1024 * 500];
			int bytes_readed = 0;
			while ((bytes_readed = finput.read(buffer)) > 0)
				foutput.write(buffer, 0, bytes_readed);
			finput.close();
			foutput.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
}
